"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var joi_1 = __importDefault(require("@hapi/joi"));
var transformCSS_1 = require("./transformCSS");
var cssPropertiesSchema = joi_1.default
    .object()
    .pattern(joi_1.default.string().regex(/^[^:@]/), [joi_1.default.string(), joi_1.default.number()]);
var keyframesSchema = joi_1.default.object().pattern(joi_1.default.string(), cssPropertiesSchema);
var cssWithKeyframesSchema = cssPropertiesSchema.append({
    '@keyframes': [joi_1.default.string(), keyframesSchema],
});
var cssWithSimplePseudos = cssWithKeyframesSchema.append(Object.assign.apply(Object, [{}].concat(transformCSS_1.simplePseudos.map(function (pseudo) {
    var _a;
    return (_a = {}, _a[pseudo] = cssWithKeyframesSchema, _a);
}))));
var selectorsSchema = joi_1.default
    .object()
    .pattern(joi_1.default.string(), cssWithKeyframesSchema);
var fullStyle = cssWithSimplePseudos.append({
    selectors: selectorsSchema,
});
var makeMediaQuerySchema = function (valueSchema) {
    return valueSchema.append({
        '@media': joi_1.default.object().pattern(joi_1.default.string(), valueSchema),
    });
};
var fullStyleWithMedia = makeMediaQuerySchema(fullStyle).required();
var globalStyleWithMedia = makeMediaQuerySchema(fullStyle.forbiddenKeys.apply(fullStyle, ['selectors'].concat(transformCSS_1.simplePseudos))).required();
var validate = function (handler) { return function (value) {
    var error = handler.validate(value).error;
    if (error !== null) {
        throw error;
    }
}; };
exports.validateGlobalStyle = validate(globalStyleWithMedia);
exports.validateStyle = validate(fullStyleWithMedia);
